/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2008 Bull S.A.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * --------------------------------------------------------------------------
 * $Id: Launcher.java 4389 2008-12-15 13:48:57Z alitokmen $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.launcher;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.apache.felix.framework.Felix;
import org.apache.felix.framework.cache.BundleCache;
import org.apache.felix.framework.util.StringMap;
import org.apache.felix.main.AutoActivator;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleException;

/**
 * Implement a generic launcher for felix based applications.
 * @author Benjamin Strappazzon
 */
public class Launcher implements ILauncher {

    /**
     * Properties to define which bundles start with the gateway. It also define
     * the startlevel for each bundles.
     */
    protected Properties autoloadConfigurationFile = null;

    /**
     * Felix configuration file.
     */
    protected Properties felixConfig = null;

    /**
     * A list of bundle which won't start with the gateway.
     */
    protected List<String> excludedBundles = null;

    /**
     * The path of the maven repository.
     */
    protected String mavenRepository = null;

    /**
     * Location of the felix cache.
     */
    protected String felixCacheDirectory;

    /**
     * Felix instance.
     */
    protected Felix felix;


    /**
     * Specify if the bundle cache must be cleaned before started the gateway.
     */
    protected boolean forceCacheCleanup = false;

    /**
     * Set the properties which lists bundles that must start with felix and
     * their start level.
     * @param autoloadProperties auto load properties.
     */
    public void setAutoloadConfigurationFile(final Properties autoloadProperties) {
        this.autoloadConfigurationFile = autoloadProperties;
    }

    /**
     * Set the configuration of felix gateway. If none is provided, a default
     * one is used.
     * @param config the configuration properties.
     */
    public void setConfigProperties(final Properties config) {
        this.felixConfig = config;
    }

    /**
     * @param excludedBundles A set of bundle to be excluded from the auto load
     *        list.
     */
    public void setExcludedBundles(final List<String> excludedBundles) {
        this.excludedBundles = excludedBundles;
    }

    /**
     * Configure and start the gateway.
     * @throws BundleException Thrown if an error occurs during starting.
     */
    public void start() throws BundleException {

        this.init();
        // read configuration files and setup Felix framework
        this.configureFelix();
        this.felix.start();
    }

    /**
     * Stop Felix.
     */
    public void stop() throws BundleException {
     // Only stop if the system is still running
        if (felix.getState() == Bundle.ACTIVE) {
            felix.stop();
        }
    }

    /**
     * Set the maven repository path.
     * @param Maven repository path.
     */
    public void setMavenRepositoryRoot(final String repositoryPath) {
        this.mavenRepository = repositoryPath;
    }

    /**
     * Nothing to do by default.
     */
    public void init() {
        // add customizations
    }

    /**
     * Retrieve the configuration and construct felix instance.
     */
    private void configureFelix() {

        //compute configuration using specific settings.
        Map prop = this.getFelixConfig();

     // Delete the cache directory if requested
        if (this.forceCacheCleanup) {
            deleteFileOrDir(new File((String) prop.get(BundleCache.CACHE_PROFILE_DIR_PROP)));
        }


        List<BundleActivator> activators = new ArrayList<BundleActivator>();
        // construct an instance of felix using the specified properties and
        // activators
        activators.add(new AutoActivator(prop));

        Map configMap = new StringMap(prop, false);

        this.felix = new Felix(configMap, activators);
    }

    /**
     * Compute the configuration using the provided specific settings.
     * @return the configuration computed by the {@link ConfigurationProvider}.
     */
    private Map getFelixConfig() {
        Map maconfig = null;
        try {
            ConfigurationProvider p = new ConfigurationProvider(this.autoloadConfigurationFile, this.excludedBundles,
                    this.felixConfig, this.mavenRepository, this.felixCacheDirectory);
            maconfig = p.getConfiguration();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return maconfig;
    }

    /**
     * Set the felix bundle cache directory.
     * @param cacheDirectory The new felixCacheDirectory.
     */
    public void setFelixCacheDirectory(final String cacheDirectory) {
        this.felixCacheDirectory = cacheDirectory;

    }

    /**
     * Force cache deletion on startup.
     */
    public void forceCacheCleanup() {
       this.forceCacheCleanup = true;
    }

    /**
     * Utility method used to delete the profile directory when run as a
     * stand-alone application.
     * @param file The file to recursively delete.
     */
    private static void deleteFileOrDir(final File file) {
        if (file.isDirectory()) {
            File[] childs = file.listFiles();
            for (int i = 0; i < childs.length; i++) {
                deleteFileOrDir(childs[i]);
            }
        }
        file.delete();
    }


}
